/**
 * ------------------------------------------------------------------------
 * This is free and unencumbered software released into the public domain.
 *
 * Anyone is free to copy, modify, publish, use, compile, sell, or
 * distribute this software, either in source code form or as a compiled
 * binary, for any purpose, commercial or non-commercial, and by any
 * means.
 *
 * In jurisdictions that recognize copyright laws, the author or authors
 * of this software dedicate any and all copyright interest in the
 * software to the public domain. We make this dedication for the benefit
 * of the public at large and to the detriment of our heirs and
 * successors. We intend this dedication to be an overt act of
 * relinquishment in perpetuity of all present and future rights to this
 * software under copyright law.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * For more information, please refer to <http://unlicense.org/>
 * ------------------------------------------------------------------------
 * Dropper.cpp -- The Ares Project, Deimos v1.0.0
 * ------------------------------------------------------------------------
 */
#include <Arduino.h>
#include "Dropper.h"
#include "DropperServo.h"

/**
 * Dropper constructor, for a dropper with num_servos servos attached.
 * No safeguards for num_servos > length of arrays.
 *
 * @param num_servos Number of distinct servos attached to dropper
 * construction (int).
 * @param pins Array of pin numbers (int*).
 * @param drop_thetas Array of absolute angles for drop position for each
 * servo (int*).
 * @param reset_thetas Array of absolute angles for the baseline or reset
 * position for each servo (int*).
 */
Dropper::Dropper(int num_servos,
                 int *pins,
                 servo_angle_t *drop_thetas,
                 servo_angle_t *reset_thetas) {
    _active_servo = SERVO_ONE;
    _num_servos = num_servos;
    for (int i = 0; i < _num_servos; i++) {
        _servos[i] = new DropperServo(
            pins[i],
            drop_thetas[i],
            reset_thetas[i]
        );
    }
}

/**
 * Initialize the dropper construction, individually initializing the
 * attached servos.
 */
void Dropper::init() {
    for (int i = 0; i < _num_servos; i++) {
        _servos[i]->servo_init();
    }
}

/**
 * Set the active servo to the drop position.
 */
void Dropper::drop() {
    _servos[_active_servo]->servo_drop();
}

/**
 * Set all servos to the drop position.
 */
void Dropper::drop_all() {
    for (int i = 0; i < _num_servos; i++) {
        _servos[i]->servo_drop();
    }
}

/**
 * Set the active servo to the reset position.
 */
void Dropper::reset() {
    _servos[_active_servo]->servo_reset();
}

/**
 * Set all servos to the reset position.
 */
void Dropper::reset_all() {
    for (int i = 0; i < _num_servos; i++) {
        _servos[i]->servo_reset();
    }
}

/**
 * Raw write an angle, specified as theta, to the servo specified.
 *
 * @param theta The angle to set (int).
 * @param servo_num The servo to set to angle theta (servo_number_t).
 */
void Dropper::write(servo_angle_t theta,
                    servo_number_t servo_num) {
    _servos[servo_num]->servo_write(theta);
}

/**
 * Advance the active servo.
 *
 * @return The new active servo as servo_number_t.
 */
servo_number_t Dropper::next() {
    int next = (int)_active_servo + 1;
    if (next >= _num_servos) {
        _active_servo = SERVO_ONE;
    } else {
        _active_servo = (servo_number_t)next;
    }
    return _active_servo;
}

/**
 * Get the active dropper.
 *
 * @return Pointer to DropperServo for active dropper.
 */
DropperServo* Dropper::active_dropper() {
    return _servos[_active_servo];
}
