/**
 * ------------------------------------------------------------------------
 * This is free and unencumbered software released into the public domain.
 *
 * Anyone is free to copy, modify, publish, use, compile, sell, or
 * distribute this software, either in source code form or as a compiled
 * binary, for any purpose, commercial or non-commercial, and by any
 * means.
 *
 * In jurisdictions that recognize copyright laws, the author or authors
 * of this software dedicate any and all copyright interest in the
 * software to the public domain. We make this dedication for the benefit
 * of the public at large and to the detriment of our heirs and
 * successors. We intend this dedication to be an overt act of
 * relinquishment in perpetuity of all present and future rights to this
 * software under copyright law.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * For more information, please refer to <http://unlicense.org/>
 * ------------------------------------------------------------------------
 * K.ino -- The Ares Project, Deimos v1.0.0
 * This file contains the Arduino sketch required to run any of the
 * supported droppers released in the file pack. Uses the lib defined by
 * two pairs of .cpp and .h files, for managing dropper.
 * ------------------------------------------------------------------------
 */
#include "Dropper.h"

/**
 * Preprocessor macros for controlling which dropper variant you are
 * using:
 *   - Enable the DOUBLE_DROPPER line if there are two dropper servos.
 *   - If DOUBLE_DROPPER is disabled only one servo will be used.
 *   - If DROP_ALL is on (with DOUBLE_DROPPER), drop both items at
 *     the same time with a single trigger.
 *   - By default, with DROP_ALL off, the two drop servos will triggered
 *     individually in sequence.
 * By default, the double dropper is used since in field conditions, it will
 * work with no changes to the macros for everything in this release.
 */
#define DOUBLE_DROPPER
#ifdef DOUBLE_DROPPER
    // #define DROP_ALL
#endif

#ifdef DOUBLE_DROPPER
const int NUM_SERVOS = 2;
#else
const int NUM_SERVOS = 1;
#endif

int SERVO_PINS[NUM_SERVOS] = {
    3,    // SERVO1
    #ifdef DOUBLE_DROPPER
    5,    // SERVO2
    #endif
};

servo_angle_t SERVO_DROP_ANGLES[NUM_SERVOS] = {
    100,  // SERVO1
    #ifdef DOUBLE_DROPPER
    80,   // SERVO2
    #endif
};

servo_angle_t SERVO_RESET_ANGLES[NUM_SERVOS] = {
    0,    // SERVO1
    #ifdef DOUBLE_DROPPER
    180,  // SERVO2
    #endif
};

const int PR_INPUT_PIN = A0;     // Photoresistor input on Analog0
const int MIN_PR_READ = 500;     // Reduce if LEDs arent bright enough,
                                 // Increase if it is being triggered on accident.
const int NON_PR_INPUT_PIN = 2;  // Digital 2
const int BASE_DELAY_MS = 2;     // Delay ms on each loop
const int DROP_DELAY_MS = 2000;  // Delay ms after a drop
const int PRIMING_DELAY_MS = 0;  // Delay ms after turning the system on
// If your drone automatically turns on the LEDs when powering on, make sure you are
// using PRIMING_DELAY_MS

bool use_photoresistor = false;
bool reset_required = false;
Dropper dropper(
    NUM_SERVOS,
    SERVO_PINS,
    SERVO_DROP_ANGLES,
    SERVO_RESET_ANGLES
);

void setup() {
    pinMode(NON_PR_INPUT_PIN, INPUT);
    if (digitalRead(NON_PR_INPUT_PIN) != LOW) {
        pinMode(PR_INPUT_PIN, INPUT);
        use_photoresistor = true;
    }
    dropper.init();
    if (PRIMING_DELAY_MS) {
        delay(PRIMING_DELAY_MS);
    }
}

void loop() {
    int r = (use_photoresistor) ? analogRead(PR_INPUT_PIN) : digitalRead(NON_PR_INPUT_PIN);
    if (reset_required) {
        #ifdef DROP_ALL
            dropper.reset_all();
        #elif defined(DOUBLE_DROPPER)
            dropper.reset();
            dropper.next();
        #else
            dropper.reset();
        #endif
        reset_required = false;
    } else if ((use_photoresistor && r >= MIN_PR_READ) || (!use_photoresistor && r == HIGH)) {
        #ifdef DROP_ALL
            dropper.drop_all();
        #else
            dropper.drop();
        #endif
        if (DROP_DELAY_MS) {
            delay(DROP_DELAY_MS);
        }
        reset_required = true;
    }
    if (BASE_DELAY_MS) {
        delay(BASE_DELAY_MS);
    }
}
